package com.ibm.samples;
import com.ibm.broker.plugin.*;
import java.io.*;
import java.net.*;
import java.util.*;
public class FTPDNode extends MbInputNode implements MbInputNodeInterface {
	public static String _FTPPortNumber = "21";
	int FAILURE;
	//
	//
	static AcceptThread acceptThread = new AcceptThread();
	//
	private static ThreadLocal localsession = new ThreadLocal() {
		protected synchronized Object initialValue() {
			return null; // no session initially
		}
	};
	// ---------------------------------		
	// Hastable to hold the FTD command exectution singletons
	private Hashtable commands = new Hashtable();
	// ---------------------------------	
	// FTP Commands that we support
	static final String STOR_COMMAND = "STOR";
	static final int STORE = 0;
	static final String QUIT_COMMAND = "QUIT";
	static final int QUIT = 1;
	static final String CWD_COMMAND = "CWD";
	static final int CWD = 2;
	static final String TYPE_COMMAND = "TYPE";
	static final int TYPE = 3;
	static final String USER_COMMAND = "USER";
	static final int USER = 4;
	static final String PASS_COMMAND = "PASS";
	static final int PASS = 5;
	static final String PORT_COMMAND = "PORT";
	static final int PORT = 6;

	// ---------------------------------	
	// Constructor 
	public FTPDNode() throws MbException {
		// create terminals here
		createOutputTerminal("catch");		
		createOutputTerminal("out");
		createOutputTerminal("failure");
		// ---------------------------------	
		// Initialize FTP command executor table
		commands.put(STOR_COMMAND, new STORCommand());
		commands.put(QUIT_COMMAND, new QUITCommand());
		commands.put(CWD_COMMAND, new CWDCommand());
		commands.put(TYPE_COMMAND, new TYPECommand());
		commands.put(USER_COMMAND, new USERCommand());
		commands.put(PASS_COMMAND, new PASSCommand());
		commands.put(PORT_COMMAND, new PORTCommand());
		// ---------------------------------			
	}
	// ---------------------------------	
	// WMQI Node name
	public static String getNodeName() {
		return "FTPDNode";
	}
	// ---------------------------------	
	// FTP command port accessors
	public String getFtpPortNumber() {
		return _FTPPortNumber;
	}
	public void setFtpPortNumber(String portNumber) {
		_FTPPortNumber = portNumber;
	}
	// ---------------------------------	
	// WMQI input node execution loop
	public int run(MbMessageAssembly assembly) throws MbException {

		// ---------------------------------			
		// select correct routine
		try {
			runCommandSocket(assembly);
		} catch (InterruptedIOException e) {
			// System.out.println("Timed out");
			return TIMEOUT; // Was timeout
		} catch (FTPDNodeException e) {
			System.out.println("Failure ");
			return FAILURE_RETURN;
		} catch (Exception e) {
			logMessage("run", "4E", e.getMessage());
			System.out.println("\n------------\nOther Failure ");
			e.printStackTrace();
			try {
				Thread.sleep(5000);
			} catch (java.lang.InterruptedException jlie) {
			}

			return SUCCESS_RETURN;
		}
		return SUCCESS_CONTINUE;
	}

	String _messageDomain = "XML";
	String getDomain() {
		return _messageDomain;
	}
	void setDomain(String s) {
		_messageDomain = s;
	}
	String _messageSet = "";
	String getSet() {
		return _messageSet;
	}
	void setSet(String s) {
		_messageSet = s;
	}
	String _messageType = "";
	String getType() {
		return _messageType;
	}
	void setType(String s) {
		_messageType = s;
	}
	String _messageFormat = "";
	String getFormat() {
		return _messageFormat;
	}
	void setFormat(String s) {
		_messageFormat = s;
	}

	// ---------------------------------			
	// The FTPD server command processor
	private void runCommandSocket(MbMessageAssembly assembly) throws Exception {

		PrintWriter reply; // Command response
		BufferedReader in; // Command input
		int len;
		String commandLine;
		WMQI_FTPSession session;
		Socket commandSocket;
		ByteArrayOutputStream byteStream;
		boolean close = false;
		//-------------------------------------------------------------------------
		// -------------------------------------------------------------------------
		// get an FTP session for this node (Synchronized method to avoid conflicts between node instances)
		// System.out.println("get local session");
		session = (WMQI_FTPSession) localsession.get();
		if (session == null) {
			// System.out.println("local session is null wait for a new one");	
			if (acceptThread==null) {
				acceptThread = new AcceptThread();				
				return;
			} else {		
				session = acceptThread.getSession();
			}
			session.setAssembly(assembly);
			localsession.set(session);
			session.setNode(this);
			// Ensure another instance of Input node can wait for incoming FTP requests
			dispatchThread();

		}
		commandSocket = session.getCommandSocket();
		reply = session.getReply();
		if (reply == null) {
			reply =
				new PrintWriter(new OutputStreamWriter(commandSocket.getOutputStream()), true);
			session.setReply(reply);
			reply.println(
				"220 Welcome to WMQI FTP Server \nSupported commands are USER PWD CDWD and STOR");
			reply.flush();

		}
		in = session.getIn();
		if (in == null) {
			in = new BufferedReader(new InputStreamReader(commandSocket.getInputStream()));
		}
		// ------------------
		// get FTP commands
		try {
			while ((commandLine = in.readLine()) != null) { //append to input;
				// --------------------------------
				// get command word
				StringTokenizer st = new java.util.StringTokenizer(commandLine);
				if (st.hasMoreTokens()) {
					String command = st.nextToken().toUpperCase();
					int toklen = command.length();
					// --------------------------------
					// Find command executor
					FTPDCommand executor = (FTPDCommand) commands.get(command);
					if (executor != null) {
						commandLine = commandLine + "          ";
						if (!executor.execute(commandLine.substring(toklen + 1).trim(), session)) {
							break; // Commit command 
						}
					} else {
						reply.println("500 '" + command + "' not supported");
					}
				}
			}
		} catch (SocketException s) {
			System.out.println("\rSocket exception connection closed. \n");
			localsession.set(null);
			commandSocket.close();
			throw s; // rethrow
		} catch (Exception e) {
			reply.println("500  Exception " + e.toString());
			throw new FTPDNodeException();
		}

		// --------------------------------
		// Close socket if quit encountered
		if (session.getQuit()) {
			localsession.set(null);
			commandSocket.close();
			System.out.println("\rConnection closed. Message created. \n");
		}

		// --------------------------------		

	}
	public void onDelete() {
		// perform node clean up if necessary. 
		WMQI_FTPSession session;
		Socket commandSocket;
		try {
			// Cleaning 
			System.out.println("Cleaning resources");
			session = (WMQI_FTPSession) localsession.get();
			if (session != null) {
				commandSocket = session.getCommandSocket();
				commandSocket.close();
			}
			localsession.set(null);
			if (acceptThread != null) {
				acceptThread.interrupt();
				acceptThread = null;

			}
		} catch (Exception e) {
		}
		System.gc();
	}
	//---------------------------------------------
	// Message logging 
	void logMessage(String methodName, String messageID, String traceText)
		throws MbException {
		MbService mbsLog = new MbService();
		if (messageID.endsWith("W")) {
			mbsLog
				.logWarning(
					this,
					methodName,
					"com.ibm.samples.FTPDNodeMessages",
					messageID,
					traceText,
					new Object[] {
			});
		} else {
			if (messageID.endsWith("I")) {
				mbsLog
					.logInformation(
						this,
						methodName,
						"com.ibm.samples.FTPDNodeMessages",
						messageID,
						traceText,
						new Object[] {
				});
			} else {
				mbsLog
					.logError(
						this,
						methodName,
						"com.ibm.samples.FTPDNodeMessages",
						messageID,
						traceText,
						new Object[] {
				});
			}
		}
	}
}