package com.ibm.samples;
import java.io.*;
import java.net.*;
import com.ibm.broker.plugin.*;
public class STORCommand extends FTPDCommand {
	STORCommand() {
	}
	boolean execute(String parameters, WMQI_FTPSession session) throws Exception {
		PrintWriter reply = session.getReply();
		Socket dataSocket;
		boolean storOK = true;
		MbMessageAssembly outputAssembly = null;
		MbMessage sentMessage = null;
		MbOutputTerminal outTerminal = session.getNode().getOutputTerminal("out");
		if (outTerminal == null) {
			reply.println("425 Data connection refused WMQI cannot acces output terminal.");
			return false;
		}
		try {
			dataSocket = new Socket(session.getDataAddress(), session.getDataport());

		} catch (Exception e) {
			dataSocket = null;
			reply.println("425 Data socket exception");
			return false;

		}
		if (dataSocket != null) {
			ByteArrayOutputStream byteStream = session.getByteStream();
			int splitlen;
			splitlen = session.getSplit();
			if ((session.getType() == WMQI_FTPSession.TYPE_BINARY)
				&& (splitlen != WMQI_FTPSession.SPLIT_CRLF)) {
				DataInputStream dataStream = null;
				try {
					dataStream = new DataInputStream(dataSocket.getInputStream());
					reply.println("150 Opening data connection in binary mode. " + splitlen);
					// ------------------------------
					// Allocate buffer for data
					byte[] databuffer;
					int bufferlength = 0x1000;
					if (splitlen > 0) {
						bufferlength = splitlen;
					}
					databuffer = new byte[bufferlength];
					//--------------------------------
					// Now read the information and propagate
					int bytes = 0;
					int totalbytes = 0;
					while (bytes != -1) {
						//------------------------------------------------------
						// If we are spliting the file write a message per line 			
						if (splitlen > 0) {
							dataStream.readFully(databuffer);
							session.getByteStream().write(databuffer);
							outputAssembly =
								new MbMessageAssembly(
									session.getAssembly(),
									session.getNode().createMessage(databuffer));
							addMessageInfo(outputAssembly, session);
							addUserPassword(outputAssembly, session);
							sentMessage = outputAssembly.getMessage();
							outTerminal.propagate(outputAssembly);
						} else {
							bytes = dataStream.read(databuffer);
							if (bytes > 0) {
								session.getByteStream().write(databuffer, 0, bytes);
							}
						}
					}
					//------------------------------------------------------
					// If we are handling the file as a whole write it now 
					if (splitlen <= 0) {
						outputAssembly =
							new MbMessageAssembly(
								session.getAssembly(),
								session.getNode().createMessage(byteStream.toByteArray()));
						addMessageInfo(outputAssembly, session);
						addUserPassword(outputAssembly, session);

						sentMessage = outputAssembly.getMessage();
						outTerminal.propagate(outputAssembly);
						byteStream.reset();
					}
				} catch (Exception e) {
					storOK = false;
					if (dataStream == null) {
						reply.println("425 Data connection refused.");
					} else {
						reply.println("226 Transfer error " + e.toString());
					}
				}
			} else
				if (session.getType() == WMQI_FTPSession.TYPE_ASCII) {
					BufferedReader reader = null;
					try {
						reader =
							new BufferedReader(
								new InputStreamReader(new DataInputStream(dataSocket.getInputStream())));
						reply.println("150 Opening data connection in ASCII mode.");
						BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(byteStream));
						String line;
						//-------------------------
						// Read per lines in ASCII mode
						line = reader.readLine();
						while (line != null) {
							writer.write(line);
							line = reader.readLine();
							//------------------------------------------------------
							// If we are spliting the file write a message per line 			
							if (splitlen == WMQI_FTPSession.SPLIT_CRLF) {
								writer.flush();
								outputAssembly =
									new MbMessageAssembly(
										session.getAssembly(),
										session.getNode().createMessage(byteStream.toByteArray()));
								addMessageInfo(outputAssembly, session);
								addUserPassword(outputAssembly, session);
								sentMessage = outputAssembly.getMessage();
								outTerminal.propagate(outputAssembly);
								byteStream.reset();
							} else {
								//--------------------------------
								// restore the newLine for a global file except last
								if (line != null)
									writer.newLine();
							}
						}
						//------------------------------------------------------
						// If we are handling the file as a whole write it now 
						if (splitlen != WMQI_FTPSession.SPLIT_CRLF) {
							writer.flush();
							outputAssembly =
								new MbMessageAssembly(
									session.getAssembly(),
									session.getNode().createMessage(byteStream.toByteArray()));
							addMessageInfo(outputAssembly, session);
							addUserPassword(outputAssembly, session);
							sentMessage = outputAssembly.getMessage();
							outTerminal.propagate(outputAssembly);
							byteStream.reset();
						}

					} catch (Exception e) {
						storOK = false;
						if (reader == null) {
							reply.println("425 Data connection refused.");
						} else {
							if (sentMessage != null) {
								reply.println(
									"226 Transfer error "
										+ e.toString()
										+ "\n message="
										+ new String(sentMessage.getBuffer()));
							} else {
								reply.println("226 Transfer error no message" + e.toString() + "\n");
							}
						}

					}

				}
			if (storOK) {
				reply.println("");
				return false; // Stop looping for commands commit session
			}

		} else {
			reply.println("425 Data connection refused.");
		}
		return true;
	}
	void addUserPassword(
		MbMessageAssembly outputAssembly,
		WMQI_FTPSession session) {
		PrintWriter reply = session.getReply();
		try {
			MbMessage message = outputAssembly.getMessage();
			MbElement root = message.getRootElement();
			MbElement properties = root.getFirstElementByPath("Properties");
			if (properties == null) {
				properties =
					root.createElementAsLastChild(MbElement.TYPE_NAME, "Properties", null);

			}
			properties.createElementAsLastChild(
				MbElement.TYPE_NAME_VALUE,
				"userid",
				session.getUser());
			properties.createElementAsLastChild(
				MbElement.TYPE_NAME_VALUE,
				"password",
				session.getPassword());

		} catch (Exception e) {
			reply.println("220 Exception storing userid/password " + e.toString());
		}

	}
	void addMessageInfo(
		MbMessageAssembly outputAssembly,
		WMQI_FTPSession session) {
		PrintWriter reply = session.getReply();
		FTPDNode node = session.getNode();
		try {
			MbMessage message = outputAssembly.getMessage();
			MbElement root = message.getRootElement();
			MbElement properties = root.getFirstElementByPath("Properties");
			if (properties == null) {
				properties =
					root.createElementAsLastChild(MbElement.TYPE_NAME, "Properties", null);

			}
			properties.createElementAsFirstChild(
				MbElement.TYPE_NAME_VALUE,
				"MessageDomain",
				node.getDomain());
			properties.createElementAsFirstChild(
				MbElement.TYPE_NAME_VALUE,
				"MessageType",
				node.getType());
			properties.createElementAsFirstChild(
				MbElement.TYPE_NAME_VALUE,
				"MessageFormat",
				node.getFormat());
			properties.createElementAsFirstChild(
				MbElement.TYPE_NAME_VALUE,
				"MessageSet",
				node.getSet());

		} catch (Exception e) {
			reply.println("220 Exception storing userid/password " + e.toString());
		}
	}

}